#!/usr/bin/env python3
"""
Zero Day Detector
Advanced threat detection and zero-day vulnerability scanner.
Parameters:
    - target: Target system or network
    - mode: Scan mode (system, network, or both)
    - intensity: Scan intensity level
Requires: yara-python, volatility3 packages and admin/root privileges
"""
import yara
import argparse
import logging
from datetime import datetime
import json
import threading
import queue
import os
import psutil
import hashlib
import re
from collections import defaultdict
import subprocess
import time
import magic
import requests

class ZeroDayDetector:
    def __init__(self, target, mode='both', intensity='medium'):
        self.target = target
        self.mode = mode.lower()
        self.intensity = intensity.lower()
        self.results_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.setup_logging()
        self.load_rules()
        self.initialize_databases()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def load_rules(self):
        """Load YARA rules for threat detection"""
        try:
            # Default rules for zero-day detection
            rules_text = """
            rule Suspicious_Behavior {
                strings:
                    $inject1 = "VirtualAllocEx" nocase
                    $inject2 = "WriteProcessMemory" nocase
                    $inject3 = "CreateRemoteThread" nocase
                    $shell1 = "cmd.exe" nocase
                    $shell2 = "powershell" nocase
                    $shell3 = "sh -c" nocase
                    $network1 = "socket(" nocase
                    $network2 = "connect(" nocase
                    $exploit1 = "stack overflow" nocase
                    $exploit2 = "buffer overflow" nocase
                    $exploit3 = "heap spray" nocase
                condition:
                    2 of ($inject*) or
                    2 of ($shell*) or
                    all of ($network*) or
                    any of ($exploit*)
            }

            rule Anomalous_Execution {
                strings:
                    $unusual1 = "CreateProcess" nocase
                    $unusual2 = "ShellExecute" nocase
                    $unusual3 = "WinExec" nocase
                    $unusual4 = "system(" nocase
                    $unusual5 = "exec(" nocase
                condition:
                    3 of them
            }

            rule Memory_Manipulation {
                strings:
                    $mem1 = "VirtualProtect" nocase
                    $mem2 = "VirtualAlloc" nocase
                    $mem3 = "HeapAlloc" nocase
                    $mem4 = "memcpy" nocase
                    $mem5 = "strcpy" nocase
                condition:
                    3 of them
            }
            """
            self.rules = yara.compile(source=rules_text)
        except Exception as e:
            self.logger.error(f"Error loading YARA rules: {str(e)}")
            self.rules = None

    def initialize_databases(self):
        """Initialize threat intelligence databases"""
        self.known_hashes = set()
        self.known_patterns = set()
        self.ioc_database = defaultdict(list)
        
        # Load known malicious hashes
        try:
            response = requests.get(
                'https://raw.githubusercontent.com/Neo23x0/signature-base/master/iocs/hash-iocs.txt'
            )
            if response.status_code == 200:
                for line in response.text.splitlines():
                    if line.strip() and not line.startswith('#'):
                        self.known_hashes.add(line.strip().lower())
        except:
            self.logger.warning("Could not load online hash database")

    def analyze_file(self, file_path):
        """Analyze a single file for potential zero-day threats"""
        try:
            file_info = {
                'path': file_path,
                'size': os.path.getsize(file_path),
                'type': magic.from_file(file_path),
                'created': datetime.fromtimestamp(
                    os.path.getctime(file_path)
                ).isoformat(),
                'modified': datetime.fromtimestamp(
                    os.path.getmtime(file_path)
                ).isoformat(),
                'anomalies': [],
                'threat_level': 'Low'
            }

            # Calculate file hash
            with open(file_path, 'rb') as f:
                content = f.read()
                file_info['md5'] = hashlib.md5(content).hexdigest()
                file_info['sha256'] = hashlib.sha256(content).hexdigest()

            # Check against known malicious hashes
            if file_info['md5'] in self.known_hashes or \
               file_info['sha256'] in self.known_hashes:
                file_info['anomalies'].append('Matches known malicious hash')
                file_info['threat_level'] = 'Critical'

            # YARA scan
            if self.rules:
                matches = self.rules.match(file_path)
                if matches:
                    file_info['anomalies'].extend(
                        [match.rule for match in matches]
                    )
                    file_info['threat_level'] = 'High'

            # Check for suspicious patterns
            if self.intensity in ['medium', 'high']:
                with open(file_path, 'rb') as f:
                    content = f.read()
                    # Check for encoded/encrypted content
                    if b'base64' in content or b'eval(' in content:
                        file_info['anomalies'].append('Contains encoded content')
                    
                    # Check for network activity
                    if b'socket' in content or b'connect(' in content:
                        file_info['anomalies'].append('Contains network activity')
                    
                    # Check for process manipulation
                    if b'CreateProcess' in content or b'fork(' in content:
                        file_info['anomalies'].append('Process manipulation')

            # Entropy analysis for high intensity scans
            if self.intensity == 'high':
                entropy = self.calculate_entropy(content)
                if entropy > 7.0:
                    file_info['anomalies'].append('High entropy (possible encryption)')

            if file_info['anomalies']:
                self.results_queue.put(file_info)

        except Exception as e:
            self.logger.error(f"Error analyzing {file_path}: {str(e)}")

    def scan_system(self):
        """Scan system for potential zero-day threats"""
        self.logger.info("Starting system scan...")
        
        # Scan running processes
        for proc in psutil.process_iter(['pid', 'name', 'cmdline', 'connections']):
            try:
                proc_info = proc.info
                suspicious = False
                anomalies = []

                # Check for suspicious network connections
                if proc_info['connections']:
                    for conn in proc_info['connections']:
                        if conn.status == 'ESTABLISHED':
                            suspicious = True
                            anomalies.append(
                                f"Unexpected network connection: {conn.raddr}"
                            )

                # Check process memory for patterns
                if self.intensity in ['medium', 'high']:
                    try:
                        with open(f"/proc/{proc_info['pid']}/maps", 'r') as f:
                            for line in f:
                                if 'rwx' in line:  # Executable memory regions
                                    suspicious = True
                                    anomalies.append('Executable memory detected')
                                    break
                    except:
                        pass

                if suspicious:
                    self.results_queue.put({
                        'type': 'process',
                        'pid': proc_info['pid'],
                        'name': proc_info['name'],
                        'anomalies': anomalies,
                        'threat_level': 'Medium'
                    })

            except (psutil.NoSuchProcess, psutil.AccessDenied):
                continue

        # Scan critical system directories
        critical_dirs = [
            '/bin', '/sbin', '/usr/bin', '/usr/sbin',
            'C:\\Windows\\System32', 'C:\\Windows\\SysWOW64'
        ]
        
        for directory in critical_dirs:
            if os.path.exists(directory):
                for root, _, files in os.walk(directory):
                    for file in files:
                        if self.stop_event.is_set():
                            return
                        file_path = os.path.join(root, file)
                        self.analyze_file(file_path)

    def scan_network(self):
        """Scan network for potential zero-day threats"""
        self.logger.info("Starting network scan...")
        
        try:
            # Basic port scan
            for port in [21, 22, 23, 25, 80, 443, 445, 3389]:
                try:
                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    sock.settimeout(1)
                    result = sock.connect_ex((self.target, port))
                    if result == 0:
                        self.results_queue.put({
                            'type': 'port',
                            'port': port,
                            'state': 'open',
                            'anomalies': ['Potentially vulnerable service'],
                            'threat_level': 'Medium'
                        })
                    sock.close()
                except:
                    continue

            # Advanced service detection for high intensity scans
            if self.intensity == 'high':
                try:
                    nmap_output = subprocess.check_output(
                        ['nmap', '-sV', '-p-', self.target],
                        stderr=subprocess.DEVNULL
                    ).decode()
                    
                    for line in nmap_output.splitlines():
                        if 'open' in line:
                            self.results_queue.put({
                                'type': 'service',
                                'details': line,
                                'anomalies': ['Exposed service'],
                                'threat_level': 'Medium'
                            })
                except:
                    self.logger.warning("Advanced service detection failed")

        except Exception as e:
            self.logger.error(f"Error in network scan: {str(e)}")

    def calculate_entropy(self, data):
        """Calculate Shannon entropy of data"""
        if not data:
            return 0
        
        entropy = 0
        for x in range(256):
            p_x = float(data.count(x))/len(data)
            if p_x > 0:
                entropy += - p_x*math.log(p_x, 2)
        return entropy

    def save_results(self, output_file):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        scan_results = {
            'timestamp': datetime.now().isoformat(),
            'target': self.target,
            'mode': self.mode,
            'intensity': self.intensity,
            'findings': results
        }

        try:
            with open(output_file, 'w') as f:
                json.dump(scan_results, f, indent=4)
            self.logger.info(f"Results saved to {output_file}")
        except Exception as e:
            self.logger.error(f"Error saving results: {str(e)}")

    def print_summary(self):
        results = []
        while not self.results_queue.empty():
            results.append(self.results_queue.get())

        self.logger.info("\nZero Day Detection Summary:")
        
        # Count findings by threat level
        threat_levels = defaultdict(int)
        for result in results:
            threat_levels[result['threat_level']] += 1

        self.logger.info("\nFindings by Threat Level:")
        for level, count in threat_levels.items():
            self.logger.info(f"{level}: {count}")

        # Display critical and high threats
        critical_threats = [r for r in results if r['threat_level'] == 'Critical']
        high_threats = [r for r in results if r['threat_level'] == 'High']

        if critical_threats:
            self.logger.info("\nCritical Threats:")
            for threat in critical_threats:
                self.logger.info(
                    f"\nType: {threat.get('type', 'Unknown')}\n"
                    f"Location: {threat.get('path', threat.get('pid', 'Unknown'))}\n"
                    f"Anomalies: {', '.join(threat['anomalies'])}"
                )

        if high_threats:
            self.logger.info("\nHigh Threats:")
            for threat in high_threats:
                self.logger.info(
                    f"\nType: {threat.get('type', 'Unknown')}\n"
                    f"Location: {threat.get('path', threat.get('pid', 'Unknown'))}\n"
                    f"Anomalies: {', '.join(threat['anomalies'])}"
                )

    def start_scan(self, output_file=None):
        self.logger.info(f"Starting zero-day detection scan")
        self.logger.info(f"Mode: {self.mode}")
        self.logger.info(f"Intensity: {self.intensity}")
        
        try:
            if self.mode in ['system', 'both']:
                self.scan_system()
            
            if self.mode in ['network', 'both']:
                self.scan_network()
                
        except KeyboardInterrupt:
            self.logger.info("Scan interrupted by user")
        except Exception as e:
            self.logger.error(f"Error during scan: {str(e)}")
        finally:
            if output_file:
                self.save_results(output_file)
            self.print_summary()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Zero Day Detector")
    parser.add_argument("target", help="Target system or network")
    parser.add_argument("--mode", choices=['system', 'network', 'both'],
                      default='both', help="Scan mode")
    parser.add_argument("--intensity", choices=['low', 'medium', 'high'],
                      default='medium', help="Scan intensity level")
    parser.add_argument("--output", help="Output file for scan results")
    args = parser.parse_args()

    detector = ZeroDayDetector(args.target, args.mode, args.intensity)
    detector.start_scan(args.output)
